@@ -17,7 +17,7 @@
 #include "fio.h"
 #include "parse.h"
 
-static char fio_version_string[] = "fio 1.16.4";
+static char fio_version_string[] = "fio 1.16.9";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -564,8 +564,12 @@ static int parse_jobs_ini(char *file, int stonewall_flag)
 	}
 
 	string = malloc(4096);
-	name = malloc(256);
-	memset(name, 0, 256);
+
+	/*
+	 * it's really 256 + small bit, 280 should suffice
+	 */
+	name = malloc(280);
+	memset(name, 0, 280);
 
 	stonewall = stonewall_flag;
 	do {
@@ -579,8 +583,8 @@ static int parse_jobs_ini(char *file, int stonewall_flag)
 		if (is_empty_or_comment(p))
 			continue;
 		if (sscanf(p, "[%255s]", name) != 1) {
-			log_err("fio: option <%s> outside of job section\n", p);
-			continue;
+			log_err("fio: option <%s> outside of [] job section\n", p);
+			break;
 		}
 
 		global = !strncmp(name, "global", 6);
