@@ -327,6 +327,15 @@ static int fill_io_u(struct thread_data *td, struct io_u *io_u)
 
 	io_u->ddir = get_rw_ddir(td);
 
+	/*
+	 * See if it's time to switch to a new zone
+	 */
+	if (td->zone_bytes >= td->o.zone_size) {
+		td->zone_bytes = 0;
+		io_u->file->last_pos += td->o.zone_skip;
+		td->io_skip_bytes += td->o.zone_skip;
+	}
+
 	/*
 	 * No log, let the seq/rand engine retrieve the next buflen and
 	 * position.
@@ -363,17 +372,22 @@ void io_u_mark_depth(struct thread_data *td, struct io_u *io_u)
 
 	switch (td->cur_depth) {
 	default:
-		index++;
+		index = 6;
+		break;
 	case 32 ... 63:
-		index++;
+		index = 5;
+		break;
 	case 16 ... 31:
-		index++;
+		index = 4;
+		break;
 	case 8 ... 15:
-		index++;
+		index = 3;
+		break;
 	case 4 ... 7:
-		index++;
+		index = 2;
+		break;
 	case 2 ... 3:
-		index++;
+		index = 1;
 	case 1:
 		break;
 	}
@@ -382,38 +396,98 @@ void io_u_mark_depth(struct thread_data *td, struct io_u *io_u)
 	td->ts.total_io_u[io_u->ddir]++;
 }
 
-static void io_u_mark_latency(struct thread_data *td, unsigned long msec)
+static void io_u_mark_lat_usec(struct thread_data *td, unsigned long usec)
+{
+	int index = 0;
+
+	assert(usec < 1000);
+
+	switch (usec) {
+	case 750 ... 999:
+		index = 9;
+		break;
+	case 500 ... 749:
+		index = 8;
+		break;
+	case 250 ... 499:
+		index = 7;
+		break;
+	case 100 ... 249:
+		index = 6;
+		break;
+	case 50 ... 99:
+		index = 5;
+		break;
+	case 20 ... 49:
+		index = 4;
+		break;
+	case 10 ... 19:
+		index = 3;
+		break;
+	case 4 ... 9:
+		index = 2;
+		break;
+	case 2 ... 3:
+		index = 1;
+	case 0 ... 1:
+		break;
+	}
+
+	assert(index < FIO_IO_U_LAT_U_NR);
+	td->ts.io_u_lat_u[index]++;
+}
+
+static void io_u_mark_lat_msec(struct thread_data *td, unsigned long msec)
 {
 	int index = 0;
 
 	switch (msec) {
 	default:
-		index++;
+		index = 11;
+		break;
 	case 1000 ... 1999:
-		index++;
+		index = 10;
+		break;
 	case 750 ... 999:
-		index++;
+		index = 9;
+		break;
 	case 500 ... 749:
-		index++;
+		index = 8;
+		break;
 	case 250 ... 499:
-		index++;
+		index = 7;
+		break;
 	case 100 ... 249:
-		index++;
+		index = 6;
+		break;
 	case 50 ... 99:
-		index++;
+		index = 5;
+		break;
 	case 20 ... 49:
-		index++;
+		index = 4;
+		break;
 	case 10 ... 19:
-		index++;
+		index = 3;
+		break;
 	case 4 ... 9:
-		index++;
+		index = 2;
+		break;
 	case 2 ... 3:
-		index++;
+		index = 1;
 	case 0 ... 1:
 		break;
 	}
 
-	td->ts.io_u_lat[index]++;
+	assert(index < FIO_IO_U_LAT_M_NR);
+	td->ts.io_u_lat_m[index]++;
+}
+
+static void io_u_mark_latency(struct thread_data *td, unsigned long usec)
+{
+	if (usec < 1000)
+		io_u_mark_lat_usec(td, usec);
+	else
+		io_u_mark_lat_msec(td, usec / 1000);
 }
 
 /*
@@ -595,11 +669,6 @@ set_file:
 
 	assert(io_u->file->flags & FIO_FILE_OPEN);
 
-	if (td->zone_bytes >= td->o.zone_size) {
-		td->zone_bytes = 0;
-		f->last_pos += td->o.zone_skip;
-	}
-
 	if (io_u->ddir != DDIR_SYNC) {
 		if (!io_u->buflen) {
 			put_io_u(td, io_u);
@@ -649,7 +718,7 @@ void io_u_log_error(struct thread_data *td, struct io_u *io_u)
 static void io_completed(struct thread_data *td, struct io_u *io_u,
 			 struct io_completion_data *icd)
 {
-	unsigned long msec;
+	unsigned long usec;
 
 	assert(io_u->flags & IO_U_F_FLIGHT);
 	io_u->flags &= ~IO_U_F_FLIGHT;
@@ -673,11 +742,11 @@ static void io_completed(struct thread_data *td, struct io_u *io_u,
 
 		io_u->file->last_completed_pos = io_u->endpos;
 
-		msec = mtime_since(&io_u->issue_time, &icd->time);
+		usec = utime_since(&io_u->issue_time, &icd->time);
 
-		add_clat_sample(td, idx, msec);
+		add_clat_sample(td, idx, usec);
 		add_bw_sample(td, idx, &icd->time);
-		io_u_mark_latency(td, msec);
+		io_u_mark_latency(td, usec);
 
 		if (td_write(td) && idx == DDIR_WRITE &&
 		    td->o.verify != VERIFY_NONE)
@@ -774,7 +843,7 @@ void io_u_queued(struct thread_data *td, struct io_u *io_u)
 {
 	unsigned long slat_time;
 
-	slat_time = mtime_since(&io_u->start_time, &io_u->issue_time);
+	slat_time = utime_since(&io_u->start_time, &io_u->issue_time);
 	add_slat_sample(td, io_u->ddir, slat_time);
 }
 
